package de.mattv.fileserver.routes;

import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.PublicRestController;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.servlet.http.HttpServletResponse;
import de.mattv.fileserver.util.NonNull;
import org.springframework.http.ContentDisposition;
import org.springframework.http.MediaType;
import org.springframework.http.MediaTypeFactory;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.PostMapping;

import java.io.IOException;
import java.nio.file.Files;

@PublicRestController
public class Download {
    private record Body(@NonNull String token, @NonNull long node) {}

    @PostMapping(path = "/download", consumes = {MediaType.APPLICATION_FORM_URLENCODED_VALUE})
    @Operation(hidden = true)
    private void download(Body body, HttpServletResponse response) throws IOException {
        Token token = TokenService.getToken(body.token);
        if (token == null) {
            response.sendError(401, "Invalid token");
            return;
        }

        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(body.node);
            if (node == null || !node.isFile) {
                response.sendError(401, "Invalid node");
                return;
            }

            response.setContentType(MediaTypeFactory.getMediaType(node.name).orElse(MediaType.APPLICATION_OCTET_STREAM).toString());
            response.setContentLengthLong(node.size);
            response.setHeader("Content-Disposition", ContentDisposition.attachment().filename(node.name).build().toString());
            FileCopyUtils.copy(Files.newInputStream(node.file.toPath()), response.getOutputStream());
            response.flushBuffer();
        }
    }
}
