package de.mattv.fileserver.routes.auth;

import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.PublicRestController;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.NonNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Optional;

@PublicRestController
public class Signup {
    private record Body(@NonNull String username, @NonNull String password) {}

    @PostMapping("/auth/signup")
    private @NonNull Optional<String> signup(@RequestBody Body body) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.writeLock())) {
            if (Data.findUser(body.username) != null)
                return Optional.of("User already exists");

            if (body.password.length() < 6)
                return Optional.of("Password must be at least 6 characters");

            String hash = AuthUtils.PW_ENCODER.encode(body.password);
            long id = Data.nextUserId++;
            User user = new User(id, body.username, hash);
            Data.USERS.put(id, user);
            Data.save();
            return Optional.empty();
        }
    }
}
