package de.mattv.fileserver.routes.fs;

import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserRestController;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.io.IOException;
import java.util.Arrays;
import java.util.Objects;
import java.util.Stack;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@UserRestController
public class FsDelete {
    private record MyEmitter(SseEmitter emitter) {
        void send(String msg) {
            try { emitter.send(msg); } catch (IOException | IllegalStateException ignored) {}
        }
    }

    private static final ExecutorService DELETE_POOL = Executors.newSingleThreadExecutor();

    @PostMapping("/fs/delete")
    private @NonNull SseEmitter delete(@AuthUser User user, @RequestBody @NonNull long[] ids) {
        MyEmitter emitter = new MyEmitter(new SseEmitter());
        DELETE_POOL.execute(() -> {
            emitter.send("Waiting for lock...");
            try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.writeLock())) {
                emitter.send(" Acquired\n");
                Stack<Node> todo = new Stack<>();
                todo.addAll(Arrays.stream(ids)
                                .filter(n -> n > 0)
                                .mapToObj(user.nodes::get)
                                .filter(Objects::nonNull)
                                .toList());

                while (!todo.empty()) {
                    Node node = todo.pop();
                    String path = FsUtils.getPath(node);

                    if (!node.children.isEmpty()) {
                        todo.push(node);
                        emitter.send("Entering " + path + "\n");
                        todo.addAll(node.children);
                        node.children.clear();
                        continue;
                    }

                    if (node.isFile) {
                        emitter.send("Deleting " + path + "...");
                        if (!node.file.delete()) log.warn("Failed to delete file {}", node.file);
                        node.removePreview();
                        emitter.send(" Done\n");
                    } else {
                        emitter.send("Exiting " + path + "\n");
                    }
                    node.parent.children.remove(node);
                    node.parent = null;
                    user.nodes.remove(node.id);
                }
            }
            Data.save();
            emitter.emitter.complete();
        });
        return emitter.emitter;
    }
}
