package de.mattv.fileserver.routes.fs;

import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserRestController;
import de.mattv.fileserver.util.NonNull;
import org.springframework.http.MediaTypeFactory;
import org.springframework.util.MimeType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@UserRestController
public class Info {
    @PostMapping("/fs/node")
    private @NonNull Optional<Response.Node> nodeInfo(@AuthUser User user, @RequestBody @NonNull long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null) return Optional.empty();

            Response.Node me = Response.Node.from(
                    node,
                    node.children.stream()
                            .map(n -> Response.Node.from(n, new Response.Node[0]))
                            .toArray(Response.Node[]::new)
            );
            return Optional.of(me);
        }
    }

    @PostMapping("/fs/path")
    private @NonNull Optional<List<Response.PathSegment>> path(@AuthUser User user, @RequestBody @NonNull long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null) return Optional.empty();

            ArrayList<Response.PathSegment> segments = new ArrayList<>();
            while (node != null) {
                segments.add(new Response.PathSegment(node.name, node.isFile ? null : node.id));
                node = node.parent;
            }
            return Optional.of(segments.reversed());
        }
    }

    @PostMapping("/fs/size")
    private long size(@AuthUser User user, @RequestBody @NonNull long[] ids) {
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            return FsUtils.nodesSize(user, ids);
        }
    }

    @PostMapping("/fs/mime")
    private @NonNull Optional<String> mime(@AuthUser User user, @RequestBody @NonNull long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null || !node.isFile) return Optional.empty();
            return MediaTypeFactory.getMediaType(node.name).map(MimeType::toString);
        }
    }
}
